Package: cuda.ml
Type: Package
Title: R Interface for the RAPIDS cuML Suite of Libraries
Version: 0.3.3
Authors@R: 
    c(person(given = "Yitao",
           family = "Li",
           role = c("aut", "cph"),
           email = "yitaoli1990@gmail.com",
           comment = c(ORCID = "0000-0002-1261-905X")),
      person(given = "Tomasz",
             family = "Kalinowski",
             role = c("aut", "cre", "cph"),
             email = "tomasz@posit.co"),
      person(given = "Daniel",
             family = "Falbel",
             role = c("aut", "cph"),
             email = "daniel@posit.co"),
      person(given = "RStudio",
           role = c("cph", "fnd")))
Description: R interface for RAPIDS cuML (<https://github.com/rapidsai/cuml>),
    a suite of GPU-accelerated machine learning libraries powered by CUDA
    (<https://en.wikipedia.org/wiki/CUDA>).
License: MIT + file LICENSE
URL: https://mlverse.github.io/cuda.ml/
BugReports: https://github.com/mlverse/cuda.ml/issues
Depends: R (>= 3.2)
Imports: ellipsis, hardhat, parsnip, Rcpp (>= 1.0.6), rlang (>= 0.1.4)
Suggests: callr, glmnet, MASS, magrittr, mlbench, purrr, reticulate,
        testthat, xgboost
LinkingTo: Rcpp
Encoding: UTF-8
RoxygenNote: 7.3.3
OS_type: unix
SystemRequirements: RAPIDS cuML (see https://rapids.ai/start.html)
NeedsCompilation: yes
Packaged: 2026-04-29 15:32:47 UTC; dfalbel
Author: Yitao Li [aut, cph] (ORCID: <https://orcid.org/0000-0002-1261-905X>),
  Tomasz Kalinowski [aut, cre, cph],
  Daniel Falbel [aut, cph],
  RStudio [cph, fnd]
Maintainer: Tomasz Kalinowski <tomasz@posit.co>
Repository: CRAN
Date/Publication: 2026-04-29 16:10:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-04-29 20:35:36 UTC; unix
Archs: cuda.ml.so.dSYM
