# Generated by roxygen2: do not edit by hand

S3method(cuda_ml_can_predict_class_probabilities,cuda_ml_fil)
S3method(cuda_ml_can_predict_class_probabilities,cuda_ml_knn)
S3method(cuda_ml_can_predict_class_probabilities,cuda_ml_model)
S3method(cuda_ml_can_predict_class_probabilities,cuda_ml_rand_forest)
S3method(cuda_ml_can_predict_class_probabilities,default)
S3method(cuda_ml_elastic_net,data.frame)
S3method(cuda_ml_elastic_net,default)
S3method(cuda_ml_elastic_net,formula)
S3method(cuda_ml_elastic_net,matrix)
S3method(cuda_ml_elastic_net,recipe)
S3method(cuda_ml_get_state,cuda_ml_model)
S3method(cuda_ml_get_state,cuda_ml_pca)
S3method(cuda_ml_get_state,cuda_ml_rand_forest)
S3method(cuda_ml_get_state,cuda_ml_rand_proj_model)
S3method(cuda_ml_get_state,cuda_ml_svc)
S3method(cuda_ml_get_state,cuda_ml_svc_ovr)
S3method(cuda_ml_get_state,cuda_ml_svr)
S3method(cuda_ml_get_state,cuda_ml_umap)
S3method(cuda_ml_get_state,default)
S3method(cuda_ml_inverse_transform,cuda_ml_pca)
S3method(cuda_ml_inverse_transform,cuda_ml_tsvd)
S3method(cuda_ml_is_classifier,cuda_ml_model)
S3method(cuda_ml_is_classifier,default)
S3method(cuda_ml_knn,data.frame)
S3method(cuda_ml_knn,default)
S3method(cuda_ml_knn,formula)
S3method(cuda_ml_knn,matrix)
S3method(cuda_ml_knn,recipe)
S3method(cuda_ml_lasso,data.frame)
S3method(cuda_ml_lasso,default)
S3method(cuda_ml_lasso,formula)
S3method(cuda_ml_lasso,matrix)
S3method(cuda_ml_lasso,recipe)
S3method(cuda_ml_logistic_reg,data.frame)
S3method(cuda_ml_logistic_reg,default)
S3method(cuda_ml_logistic_reg,formula)
S3method(cuda_ml_logistic_reg,matrix)
S3method(cuda_ml_logistic_reg,recipe)
S3method(cuda_ml_ols,data.frame)
S3method(cuda_ml_ols,default)
S3method(cuda_ml_ols,formula)
S3method(cuda_ml_ols,matrix)
S3method(cuda_ml_ols,recipe)
S3method(cuda_ml_rand_forest,data.frame)
S3method(cuda_ml_rand_forest,default)
S3method(cuda_ml_rand_forest,formula)
S3method(cuda_ml_rand_forest,matrix)
S3method(cuda_ml_rand_forest,recipe)
S3method(cuda_ml_ridge,data.frame)
S3method(cuda_ml_ridge,default)
S3method(cuda_ml_ridge,formula)
S3method(cuda_ml_ridge,matrix)
S3method(cuda_ml_ridge,recipe)
S3method(cuda_ml_serialize,cuda_ml_model)
S3method(cuda_ml_serialize,default)
S3method(cuda_ml_set_state,cuda_ml_model_state)
S3method(cuda_ml_set_state,cuda_ml_pca_model_state)
S3method(cuda_ml_set_state,cuda_ml_rand_forest_model_state)
S3method(cuda_ml_set_state,cuda_ml_rand_proj_model_state)
S3method(cuda_ml_set_state,cuda_ml_svc_model_state)
S3method(cuda_ml_set_state,cuda_ml_svc_ovr_model_state)
S3method(cuda_ml_set_state,cuda_ml_svr_model_state)
S3method(cuda_ml_set_state,cuda_ml_umap_model_state)
S3method(cuda_ml_set_state,default)
S3method(cuda_ml_sgd,data.frame)
S3method(cuda_ml_sgd,default)
S3method(cuda_ml_sgd,formula)
S3method(cuda_ml_sgd,matrix)
S3method(cuda_ml_sgd,recipe)
S3method(cuda_ml_svm,data.frame)
S3method(cuda_ml_svm,default)
S3method(cuda_ml_svm,formula)
S3method(cuda_ml_svm,matrix)
S3method(cuda_ml_svm,recipe)
S3method(cuda_ml_transform,cuda_ml_rand_proj_model)
S3method(cuda_ml_transform,cuda_ml_tsvd)
S3method(cuda_ml_transform,cuda_ml_umap)
S3method(predict,cuda_ml_fil)
S3method(predict,cuda_ml_knn)
S3method(predict,cuda_ml_linear_model)
S3method(predict,cuda_ml_logistic_reg)
S3method(predict,cuda_ml_rand_forest)
S3method(predict,cuda_ml_svm)
export(cuML_major_version)
export(cuML_minor_version)
export(cuda_ml_agglomerative_clustering)
export(cuda_ml_can_predict_class_probabilities)
export(cuda_ml_dbscan)
export(cuda_ml_elastic_net)
export(cuda_ml_fil_enabled)
export(cuda_ml_fil_load_model)
export(cuda_ml_inverse_transform)
export(cuda_ml_is_classifier)
export(cuda_ml_kmeans)
export(cuda_ml_knn)
export(cuda_ml_knn_algo_ivfflat)
export(cuda_ml_knn_algo_ivfpq)
export(cuda_ml_knn_algo_ivfsq)
export(cuda_ml_lasso)
export(cuda_ml_logistic_reg)
export(cuda_ml_ols)
export(cuda_ml_pca)
export(cuda_ml_rand_forest)
export(cuda_ml_rand_proj)
export(cuda_ml_ridge)
export(cuda_ml_serialise)
export(cuda_ml_serialize)
export(cuda_ml_sgd)
export(cuda_ml_svm)
export(cuda_ml_transform)
export(cuda_ml_tsne)
export(cuda_ml_tsvd)
export(cuda_ml_umap)
export(cuda_ml_unserialise)
export(cuda_ml_unserialize)
export(has_cuML)
import(Rcpp)
importFrom(ellipsis,check_dots_used)
importFrom(rlang,"%||%")
importFrom(stats,predict)
useDynLib(cuda.ml, .registration = TRUE)
