# A list containing libcuml download links for "cuml_versions" and CUDA major versions.
libcuml_versions <- list(
  "21.08" = list(
    "11" = "https://github.com/mlverse/libcuml-builds/releases/download/v21.08-cuda11.2.1/libcuml-21.08-cuda11.2.1.zip"
  ),
  "21.10" = list(
    "11" = "https://github.com/mlverse/libcuml-builds/releases/download/v21.10-cuda11.2.1/libcuml-21.10-cuda11.2.1.zip"
  ),
  "21.12" = list(
    "11" = "https://github.com/mlverse/libcuml-builds/releases/download/v21.12-cuda11.2.1/libcuml-21.12-cuda11.2.1.zip"
  )
)
