% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{cuda_ml_inverse_transform}
\alias{cuda_ml_inverse_transform}
\title{Apply the inverse transformation defined by a trained cuML model.}
\usage{
cuda_ml_inverse_transform(model, x, ...)
}
\arguments{
\item{model}{A model object.}

\item{x}{The dataset to be transformed.}

\item{...}{Additional model-specific parameters (if any).}
}
\value{
The transformed data points.
}
\description{
Given a trained cuML model, apply the inverse transformation defined by that
model to an input dataset.
}
