% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{cuda_ml_is_classifier}
\alias{cuda_ml_is_classifier}
\title{Determine whether a CuML model is a classifier.}
\usage{
cuda_ml_is_classifier(model)
}
\arguments{
\item{model}{A trained CuML model.}
}
\value{
A logical value indicating whether the model is a classifier.
}
\description{
Given a trained CuML model, return \code{TRUE} if the model is a classifier,
otherwise \code{FALSE} (e.g., if the model is a regressor).
}
