% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{cuda.ml-package}
\alias{cuda.ml}
\alias{cuda.ml-package}
\title{cuda.ml}
\description{
This package provides a R interface for the RAPIDS cuML library.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://mlverse.github.io/cuda.ml/}
  \item Report bugs at \url{https://github.com/mlverse/cuda.ml/issues}
}

}
\author{
Yitao Li <yitao@rstudio.com>
}
