% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{cuda_ml_can_predict_class_probabilities}
\alias{cuda_ml_can_predict_class_probabilities}
\title{Determine whether a CuML model can predict class probabilities.}
\usage{
cuda_ml_can_predict_class_probabilities(model)
}
\arguments{
\item{model}{A trained CuML model.}
}
\value{
A logical value indicating whether the model supports outputting
  class probabilities.
}
\description{
Given a trained CuML model, return \code{TRUE} if the model is a classifier
and is capable of outputting class probabilities as prediction results (e.g.,
if the model is a KNN or an ensemble classifier), otherwise return
\code{FALSE}.
}
