% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cuml_utils.R
\name{has_cuML}
\alias{has_cuML}
\title{Determine whether \{cuda.ml\} was linked to a valid version of the RAPIDS cuML
shared library.}
\usage{
has_cuML()
}
\value{
A logical value indicating whether the current installation \{cuda.ml\}
  was linked to a valid version of the RAPIDS cuML shared library.
}
\description{
Determine whether \{cuda.ml\} was linked to a valid version of the RAPIDS cuML
shared library.
}
\examples{

library(cuda.ml)

if (!has_cuML()) {
  warning(
    "Please install the RAPIDS cuML shared library first, and then re-",
    "install {cuda.ml}."
  )
}
}
