% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knn.R
\name{cuda_ml_knn_algo_ivfflat}
\alias{cuda_ml_knn_algo_ivfflat}
\title{Build a specification for the "ivfflat" KNN query algorithm.}
\usage{
cuda_ml_knn_algo_ivfflat(nlist, nprobe)
}
\arguments{
\item{nlist}{Number of cells to partition dataset into.}

\item{nprobe}{At query time, the number of cells used for approximate nearest
neighbor search.}
}
\value{
An object encapsulating all required parameters of the "ivfflat" KNN
  query algorithm.
}
\description{
Build a specification of the flat-inverted-file KNN query algorithm, with all
required parameters specified explicitly.
}
