% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/model.R
\name{cuda_ml_serialize}
\alias{cuda_ml_serialize}
\alias{cuda_ml_serialise}
\title{Serialize a CuML model}
\usage{
cuda_ml_serialize(model, connection = NULL, ...)

cuda_ml_serialise(model, connection = NULL, ...)
}
\arguments{
\item{model}{The model object.}

\item{connection}{An open connection or \code{NULL}. If \code{NULL}, then the
model state is serialized to a raw vector. Default: NULL.}

\item{...}{Additional arguments to \code{base::serialize()}.}
}
\value{
\code{NULL} unless \code{connection} is \code{NULL}, in which case
  the serialized model state is returned as a raw vector.
}
\description{
Given a CuML model, serialize its state into a connection.
}
\seealso{
\code{\link[base]{serialize}}
}
