% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{cuda.ml}
\alias{cuda.ml}
\title{cuda.ml}
\description{
This package provides a R interface for the RAPIDS cuML library.
}
\author{
Yitao Li <yitao@rstudio.com>
}
