% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logistic_reg.R
\name{predict.cuda_ml_logistic_reg}
\alias{predict.cuda_ml_logistic_reg}
\title{Make predictions on new data points.}
\usage{
\method{predict}{cuda_ml_logistic_reg}(object, x, ...)
}
\arguments{
\item{object}{A trained CuML model.}

\item{x}{A matrix or dataframe containing new data points.}

\item{...}{Additional arguments to \code{predict()}. Currently unused.}
}
\value{
Predictions on new data points.
}
\description{
Make predictions on new data points using a CuML logistic regression model.
}
